#include "editor.h"

#define PROPERTIES_X 10
#define PROPERTIES_Y 20
#define PROPERTIES_WIDTH 140
#define PROPERTIES_HEIGHT 70
#define LABEL_WIDTH 35

#define PROPERTY_MAX_LENGTH 64
char property_name[PROPERTY_MAX_LENGTH];
char property_author[PROPERTY_MAX_LENGTH];
char property_hint[PROPERTY_MAX_LENGTH];
char property_congrat[PROPERTY_MAX_LENGTH];

#define NUM_WIDGETS 4
struct widget widgets[NUM_WIDGETS] = {
	{ 2, 14, 135, 9, 0, draw_textbox, key_textbox, "Name",    property_name },
	{ 2, 24, 135, 9, 0, draw_textbox, key_textbox, "Author",  property_author},
	{ 2, 34, 135, 9, 0, draw_textbox, key_textbox, "Hint",    property_hint  },
	{ 2, 44, 135, 9, 0, draw_textbox, key_textbox, "Congrats",property_congrat},
	};
int current_widget = 0;

//{{{
void enter_properties_state(void)
{
	input_state = properties_state;
	strcpy(property_name, current_map->name);
	strcpy(property_author, current_map->author);
	strcpy(property_hint, current_map->hint);
	strcpy(property_congrat, current_map->congrat);
	draw_current_map();
	draw_rect(PROPERTIES_X, PROPERTIES_Y, PROPERTIES_WIDTH, PROPERTIES_HEIGHT);
	draw_string("Map properties", PROPERTIES_X + 3, PROPERTIES_Y + 2);
	draw_line(PROPERTIES_X, PROPERTIES_Y+8,
	          PROPERTIES_X+PROPERTIES_WIDTH-1, PROPERTIES_Y+8, A_NORMAL);
	current_widget = 0;
	widgets[0].focused = 1;
	redraw_properties();
}
//}}}
//{{{
void redraw_properties(void)
{
	int ii;
	for(ii=0; ii<NUM_WIDGETS; ii++)
		widgets[ii].draw(&widgets[ii]);
}
//}}}
//{{{
void properties_state(int key)
{
	if(key == KEY_ENTER) {
		apply_property_changes();
		widgets[current_widget].focused = 0;
		enter_edit_state();
	} else if(key == KEY_ESC) {
		widgets[current_widget].focused = 0;
		enter_edit_state();
	} else if(key == KEY_UP) {
		if(current_widget > 0)
			switch_focus(current_widget-1);
		else
			switch_focus(NUM_WIDGETS-1);
	} else if(key == KEY_DOWN) {
		switch_focus((current_widget+1) % NUM_WIDGETS);
	} else {
		widgets[current_widget].key(&widgets[current_widget], key);
	}
}
//}}}
//{{{
void apply_property_changes(void)
{
	strcpy(current_map->name, property_name);
	strcpy(current_map->author, property_author);
	strcpy(current_map->hint, property_hint);
	strcpy(current_map->congrat, property_congrat);
}
//}}}

//{{{
void switch_focus(int focus)
{
	widgets[current_widget].focused = 0;
	widgets[current_widget].draw( &widgets[current_widget] );
	current_widget = focus;
	widgets[current_widget].focused = 1;
	widgets[current_widget].draw( &widgets[current_widget] );
}
//}}}
//{{{
void key_textbox(struct widget *this, int key)
{
	char *str = (char*)this->content;
	
	if(key == KEY_LEFT || key == KEY_BACKSPACE)
	{
		if(strlen(str) > 0)
			str[ strlen(str)-1 ] = '\0';
	} else if( key < 256 ) {
		if(strlen(str)+1 < PROPERTY_MAX_LENGTH) {
			str[ strlen(str)+1 ] = '\0';
			str[ strlen(str) ] = key;
		}
	}
	draw_textbox(this);
}
//}}}
//{{{
void draw_textbox(struct widget *this)
{
	clear_rect(PROPERTIES_X+this->x, PROPERTIES_Y+this->y,
	          this->width, this->height);
	draw_rect(PROPERTIES_X+this->x+LABEL_WIDTH, PROPERTIES_Y+this->y,
	          this->width-LABEL_WIDTH, this->height);
	draw_string(this->label, this->x+PROPERTIES_X+2, this->y+PROPERTIES_Y+2);
	draw_string((const char*)(this->content),
	            this->x+PROPERTIES_X+2+LABEL_WIDTH, this->y+PROPERTIES_Y+2);
	if(this->focused) {
		xor_rect(PROPERTIES_X+this->x+LABEL_WIDTH+1, PROPERTIES_Y+this->y+1,
		         this->width-LABEL_WIDTH-2, this->height-2);
	}
}
//}}}

