#include "editor.h"

#define MIN_SIZE_X 5
#define MIN_SIZE_Y 5
#define MAX_SIZE_X 50 // Arbitrary, but to make sure it stays sane
#define MAX_SIZE_Y 40

static long new_size_x, new_size_y;

//{{{
void enter_resize_state(void)
{
	input_state = resize_state;
	new_size_x = current_map->width;
	new_size_y = current_map->height;
	draw_resize_dialog();
}
//}}}
//{{{
void draw_resize_dialog(void)
{
	char buf[64];
	draw_rect(50, 30, 60, 15);
	sprintf(buf, "%lix%li", new_size_x, new_size_y);
	draw_string_font(buf, 80-DrawStrWidth(buf, F_8x10)/2, 32, F_8x10);
}
//}}}
//{{{
void resize_state(int key)
{
	if(key == KEY_LEFT) {
		if(new_size_x > MIN_SIZE_X) {
			new_size_x--;
			draw_resize_dialog();
		}
	} else if(key == KEY_UP) {
		if(new_size_y > MIN_SIZE_Y) {
			new_size_y--;
			draw_resize_dialog();
		}
	} else if(key == KEY_RIGHT) {
		if(new_size_x < MAX_SIZE_X) {
			new_size_x++;
			draw_resize_dialog();
		}
	} else if(key == KEY_DOWN) {
		if(new_size_y < MAX_SIZE_Y) {
			new_size_y++;
			draw_resize_dialog();
		}
	} else if(key == KEY_ENTER) { // (Apply changes)
		adjust_map_size(new_size_x, new_size_y);
		enter_edit_state();
	} else if(key == KEY_ESC) { // (Cancel)
		enter_edit_state();
	}
}
//}}}
//{{{
void adjust_map_size(int width, int height)
{
	int xi, yi;
	int xmax, ymax;
		// Nifty GCC extension
	map_square saved_data[current_map->height][current_map->width];
	
	if(cursor_x >= width) move_cursor( width-cursor_x-1, 0, 0 );
	if(cursor_y >= height) move_cursor( 0, height-cursor_y-1, 0 );
	
	xmax = current_map->width;
	ymax = current_map->height;
	
	for(yi=0; yi<ymax; yi++)
	for(xi=0; xi<xmax; xi++) {
		saved_data[yi][xi] = current_map->t[yi][xi];
	}
	
	if(xmax > width) xmax = width;
	if(ymax > height) ymax = height;
	
	set_level_size(current_map_num, width, height);
	
	for(yi=0; yi<ymax; yi++)
	for(xi=0; xi<xmax; xi++) {
		current_map->t[yi][xi] = saved_data[yi][xi];
	}
}
//}}}

void clip_scroll_pos(void)
{
	if(scroll_x + VIEWPORT_WIDTH >= current_map->width)
		scroll_x = current_map->width - VIEWPORT_WIDTH;
	if(scroll_y + VIEWPORT_HEIGHT >= current_map->height)
		scroll_y = current_map->height - VIEWPORT_HEIGHT;
	if(scroll_x < 0) scroll_x = 0;
	if(scroll_y < 0) scroll_y = 0;
}
