#include "editor.h"

#define GrayIdle() pokeIO(0x600005,0b10111)
//{{{
int read_char(void)
{
	// NOTE: The contents of this function have a large effect on battery
	// usage.
	unsigned short key;
	void *kbq = kbd_queue();
	
	OSTimerRestart(APD_TIMER);
	
	if(GrayCheckRunning())
	{
		while(OSdequeue (&key, kbq)) {
			GrayIdle();
			if(OSTimerExpired(APD_TIMER)) {
				off();
				OSTimerRestart(APD_TIMER);
			}
		}
	} else {
		while(OSdequeue (&key, kbq)) {
			idle();
			if(OSTimerExpired(APD_TIMER)) {
				off();
				OSTimerRestart(APD_TIMER);
			}
		}
	}
	return key & ~0x800;
}
//}}}

//{{{
void clear_screen(void)
{
	if( GrayCheckRunning() ) {
		GraySetAMSPlane(LIGHT_PLANE);
		clrscr();
		GraySetAMSPlane(DARK_PLANE);
	}
	clrscr();
}
//}}}
//{{{
void clear_line_range(int top, int bottom)
{
	if(GrayCheckRunning())
	{
		memset((char*)GrayGetPlane(LIGHT_PLANE) + top*30, 0, (bottom-top)*30);
		memset((char*)GrayGetPlane(DARK_PLANE) + top*30, 0, (bottom-top)*30);
	}
	memset((char*)LCD_MEM + top*30, 0, (bottom-top)*30);
}
//}}}
//{{{
/// draw_sprite
/// Draws a sprite *horizontally aligned on an 8-pixel boundary*. If the
/// x position given is not aligned, it is rounded down to the nearest
/// multiple of 8.
void draw_sprite(const char *spr, int x, int y)
{
	char *lightpos, *darkpos;
	int ii;
	
	x >>= 3;
	
	if( GrayCheckRunning() ) {
		lightpos = (char*)GrayGetPlane(LIGHT_PLANE) + y*30 + x;
		darkpos  = (char*)GrayGetPlane(DARK_PLANE)  + y*30 + x;
		spr += 8;
		for(ii=0; ii<8; ii++) {
			*darkpos  = *(spr++);
			*lightpos = *(spr++);
			darkpos  += 30;
			lightpos += 30;
		}
	} else {
		lightpos = LCD_MEM + y*30 + x;
		for(ii=0; ii<8; ii++) {
			*lightpos = *(spr++);
			lightpos += 30;
		}
	}
}
//}}}
//{{{
/// draw_sprite_xor
/// XORs a sprite *horizontally aligned on an 8-pixel boundary*. If the
/// x position given is not aligned, it is rounded down to the nearest
/// multiple of 8.
void draw_sprite_xor(const char *spr, int x, int y)
{
	char *lightpos, *darkpos;
	int ii;
	
	x >>= 3;
	
	if( GrayCheckRunning() ) {
		lightpos = (char*)GrayGetPlane(LIGHT_PLANE) + y*30 + x;
		darkpos  = (char*)GrayGetPlane(DARK_PLANE)  + y*30 + x;
		spr += 8;
		for(ii=0; ii<8; ii++) {
			*darkpos  ^= *(spr++);
			*lightpos ^= *(spr++);
			darkpos  += 30;
			lightpos += 30;
		}
	} else {
		lightpos = LCD_MEM + y*30 + x;
		for(ii=0; ii<8; ii++) {
			*lightpos ^= *(spr++);
			lightpos += 30;
		}
	}
}
//}}}
//{{{
void draw_line(short x0, short y0, short x1, short y1, short Attr)
{
	if(GrayCheckRunning())
	{
		GraySetAMSPlane(LIGHT_PLANE);
		DrawLine(x0, y0, x1, y1, Attr);
		GraySetAMSPlane(DARK_PLANE);
	}
	DrawLine(x0, y0, x1, y1, Attr);
}
//}}}
//{{{
void draw_string(const char *str, short x, short y)
{
	FontSetSys(F_4x6);
	if(GrayCheckRunning())
	{
		GraySetAMSPlane(LIGHT_PLANE);
		DrawStr(x, y, str, A_NORMAL);
		GraySetAMSPlane(DARK_PLANE);
	}
	DrawStr(x, y, str, A_NORMAL);
}
//}}}
//{{{
void draw_string_font(const char *str, short x, short y, short font)
{
	FontSetSys(font);
	if(GrayCheckRunning())
	{
		GraySetAMSPlane(LIGHT_PLANE);
		DrawStr(x, y, str, A_NORMAL);
		GraySetAMSPlane(DARK_PLANE);
	}
	DrawStr(x, y, str, A_NORMAL);
}
//}}}
//{{{
void clear_rect(int left, int top, int width, int height)
{
	SCR_RECT r = {{ left, top, left+width-1, top+height-1}};
	
	if(GrayCheckRunning())
	{
		GraySetAMSPlane(LIGHT_PLANE);
		ScrRectFill(&r, &r, A_REVERSE);
		GraySetAMSPlane(DARK_PLANE);
	}
	ScrRectFill(&r, &r, A_REVERSE);
}
//}}}
//{{{
// draw_rect: Draws a rectangle with a black border and a white interior
void draw_rect(int left, int top, int width, int height)
{
	clear_rect(left, top, width, height);
	width--;
	height--;
	draw_line(left, top, left+width, top, A_NORMAL);
	draw_line(left+width, top, left+width, top+height, A_NORMAL);
	draw_line(left+width, top+height, left, top+height, A_NORMAL);
	draw_line(left, top+height, left, top, A_NORMAL);
}
//}}}
//{{{
void xor_rect(int left, int top, int width, int height)
{
	SCR_RECT r = {{ left, top, left+width-1, top+height-1}};
	
	if(GrayCheckRunning())
	{
		GraySetAMSPlane(LIGHT_PLANE);
		ScrRectFill(&r, &r, A_XOR);
		GraySetAMSPlane(DARK_PLANE);
	}
	ScrRectFill(&r, &r, A_XOR);
}
//}}}

//{{{
void debug_print(const char *fmt, ...)
{
	char buf[512];
	va_list varargs;
	va_start(varargs, fmt);
	vsprintf(buf, fmt, varargs);
	va_end(varargs);
	draw_string(buf, 20, 30);
}
//}}}

//{{{
int is_in_bounds(int x, int y)
{
	if(x<0 || y<0 || x>=FIELD_WIDTH || y>=FIELD_HEIGHT)
		return 0;
	else
		return 1;
}
//}}}

//{{{
// freadstr: Read a null-terminated string from fp. This is different from
// fgets only in what marks the end of the string.
char *freadstr(char *s, int n, FILE *fp)
{
	// Source: tigcclib, with modifications
	short c=EOF;
	char *cs=s;
	while(--n>0&&(c=fgetc(fp))!=EOF) {
		if ((*cs++=c)=='\0') break;
	}
	*cs=0;
	return ((c==EOF&&cs==s)?0:s);
}
//}}}
//{{{
// fwritestr: Write a null-terminated string to fp.
void fwritestr(const char *s, FILE *fp)
{
	const char *cs = s;
	while(*cs != '\0') {
		fputc(*cs, fp);
		cs++;
	}
	fputc('\0', fp);
}
//}}}
